/*!
    \file   df_connection.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen für die Funktionen der Geräteverbindungen und deren direktem Zugriff.
*/
#ifndef DF_CONNECTION_H
#define DF_CONNECTION_H

#include "df_defines.h"

/*! 
	\attention Diese Funktion sollten Sie nur für die Kommunikation mit AEIII+ Geräten verwenden, bei allen anderen Geräten sollten Sie DFCComOpenIV() zum öffnen einer Verbindung nutzen.
	         Die Kommunikation startet bei Verwendung mit anderen Geräten verzögert, da vorher das zugrundeliegende Kommunikationsprotokoll ermittelt werden muss.
			 
    \brief Öffnet eine serielle Schnittstelle für die Kommunikation mit einem AEIII+ über RS232/RS485. 

    Die Funktion ruft intern die Funktion DFCComOpenSerial() mit den Standardparametern
    baudRate = 38400 und timeout = 800 auf.
	
    \param[in] connectionId Zu verwendende Verbindungs-Id. Diese wird für weitere Funktionsaufrufe zur Kommunikation benötigt. Bereich [1 - 400].

    \retval 1 Serielle Schnittstelle erfolgreich geöffnet.
    \retval 0 Fehler bei öffnen der seriellen Schnittstelle.
	
	\ingroup connection
*/
DF_API_EXTERN int DF_API_CALL DFCComInit( int connectionId );

#if (DF_API_VERSION >= 0x02000300)


/*! 
    \brief Oeffnet die serielle Schnittstelle fuer RS232 / RS485 Direktkommunikation. 

    Es können zusätzliche Parameter verwendet werden. Die Baudrate ist standardmaessig
    auf 38400bd einzustellen und der Timeout liegt bei 150ms pro Lesevorgang.

    \param[in] connectionId Kommunikationskanalnummer.
    \param[in] commPort Zeiger auf String mit Namen der Schnittstelle "COM1", "COM2" usw..
    \param[in] baudRate Baudrate mit der die Schnittstelle verwendet wird, 38400bd ist Standard.
    \param[in] timeout Zu verwendender Timeout fuer Read - Befehl. (ReadFile)

    \return Kommunikationskanal oeffnen.
    \retval 1 Schnittstelle erfolgreich geoeffnet.
    \retval 0 Fehler bei Initialisierung.
*/
DF_API_EXTERN int DF_API_CALL DFCComOpenSerial( int connectionId, char *commPort, int baudRate, int timeout );

#endif


/*! 
    \brief Oeffnet die TCP/IP Schnittstelle. 

    Die Funktion ruft intern die Funktion DFCComOpenSocket() mit den Standardparametern
    iPortNum = 8000 und iTimeOut = 3000 auf.

    \param connectionId Kommunikationskanalnummer.
    \param hostAddress Zeiger auf String mit Hostnamen oder IP - Adresse in Punktnotation.

    \return Kommunikationskanal oeffnen.
    \retval 1 Schnittstelle erfolgreich geoeffnet.
    \retval 0 Fehler bei Initialisierung.
*/
DF_API_EXTERN int DF_API_CALL DFCComOpen( int connectionId, char *hostAddress );

#if (DF_API_VERSION >= 0x02000300)


/*! 
    \brief Oeffnet die TCP/IP Schnittstelle. 

    Es koennen zusaetzliche Parameter verwendet werden. Die Portnummer ist standardmaessig
    auf 8000 einzustellen und der Timeout (ein intern verwalteter) 
    liegt bei 3000 pro Lesevorgang.

    \param connectionId Kommunikationskanalnummer.
    \param hostAddress Zeiger auf String mit Hostnamen oder IP - Adresse in Punktnotation.
    \param portNumber Portnummer 
    \param timeout Zu verwendender Timeout bei Lesevorgang. (keine ms - Einheit)

    \return Kommunikationskanal oeffnen.
    \retval 1 Schnittstelle erfolgreich geoeffnet.
    \retval 0 Fehler bei Initialisierung.
*/
DF_API_EXTERN int DF_API_CALL DFCComOpenSocket( int connectionId, char *hostAddress, int portNumber, int timeout );

#endif

#if (DF_API_VERSION >= 0x02001000)


/*! 
    \brief Oeffnet die Schnittstelle fuer die Kommunikation mit PZE Master IV.

    Initialisiert den Kanal. 
    Je nach angegebenem Typ wird ein anderes Initialisierungsverfahren verwendet 
    um die entsprechend nachfolgenden Funktionsaufrufe der exportierten Funktionen 
    richtig umzusetzen.

    \param connectionId Kommunikationskanalnummer.
    \param deviceAddress Die von der DLL zu verwendende Busnummer. (Multimasteransatz)
    \param connectionType Der zu verwendende Schnittstellentyp.
                 1 -> RS232 direkt oder über Umsetzter (RS232 <-> RS485)
                 2 -> Funk über den Umsetzer (RS232 <-> BIM2) ohne eigenen Kontroller.
                 3 -> TCP / IP - Verbindung.
    \param connectionString Je nach Schnittstellentyp:
                       1,2 -> Zeiger auf Zeichenkette für die Schnittstelle. "COM1" .. "COM4".
                       3 -> Zeiger auf Zeichenkette mit IP - Adresse oder Hostname.
    \param connectionValue Je nach Schnittstellentyp:
                     1,2 -> Übertragungsrate der Schnittstelle. (Standard ist: 38400)
                     3 -> Portnummer des TCP / IP Anschlusses.
    \param connectionTimeout Je nach Schnittstellentyp:
                       1,2 ->Timeout für Lesen von Schnittstelle (Standard ist: 150)
                       3 -> Timeout für Lesen von Socket (Standard ist: 3000)

    \return Kommunikationskanal oeffnen.
    \retval 1 Schnittstelle erfolgreich geoeffnet.
    \retval 0 Fehler bei Initialisierung.
	
	\snippet examples.cpp DFCComOpenIV
*/
DF_API_EXTERN int DF_API_CALL DFCComOpenIV( int connectionId, int deviceAddress, int connectionType, char *connectionString, int connectionValue, int connectionTimeout );

#endif


/*! 
    \brief Schliessen des geoeffneten Kanals.

    Je nach dem Typ des Kanals wird entweder die serielle Schnittstelle oder
    der Socket geschlossen.

    \param connectionId Kommunikationskanalnummer.

    \return Keine.
	
	\snippet examples.cpp DFCComClose
*/
DF_API_EXTERN void DF_API_CALL DFCComClose( int connectionId );

#if (DF_API_VERSION >= 0x04030000)


/*! 
    \brief Prueft ob die Schnittstelle bereits Initialisiert oder auch geoeffnet wurde. 

    \param connectionId Kommunikationskanalnummer.

    \return Kommunikationskanal oeffnen.
    \retval -1 Kanalobjekt noch nicht verfügbar, Schnittstelle nicht initialisiert oder geoeffnet.
    \retval 0 Schnittstelle nicht geoeffnet.
    \retval 1 Schnittstelle geoeffnet.
*/
DF_API_EXTERN int DF_API_CALL DFCIsChannelOpen( int connectionId );

#endif

#if (DF_API_VERSION >= 0x02001800)
/*! 
    \brief Schreibt Zeichen in den Kanal.

    Schreiben von Daten in den Kanal.

    \param connectionId Kommunikationskanalnummer.
    \param buffer Zeiger auf Daten.
    \param toWrite Laenge der zu schreiben Daten.
    \param written Anzahl der geschriebenen Daten.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 02.00.18 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCWrite( int connectionId, char *buffer, int toWrite, int *written, int *errorNumber );


/*! 
    \brief Liest Zeichen vom Kanal.

    Lesen von Daten aus dem Kanal.

    \param connectionId Kommunikationskanalnummer.
    \param buffer Zeiger auf Daten.
    \param toRead Laenge der zu lesenden Daten.
    \param read Anzahl der gelesenen Daten.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 02.00.18 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCRead( int connectionId, char *buffer, int toRead, int *read, int *errorNumber );
#endif

// ########################################################
// #############* PRIVATE FUNKTIONEN* #################
// ########################################################

// Funktionen sind nicht dokumentiert.
// Sie werden von AESetup oder Talk verwendet.
// Unterliegen evntl. Aenderungen.

#if defined( USE_PRIVATE )


DF_API_EXTERN int DF_API_CALL DFCComUseHandle( int connectionId, HANDLE commHandle );
DF_API_EXTERN int DF_API_CALL DFCComReleaseHandle( int connectionId );

#if (DF_API_VERSION >= 0x02000300)

DF_API_EXTERN HANDLE DF_API_CALL DFCGetComPort( int connectionId );
DF_API_EXTERN int DF_API_CALL DFCSetComPort( int connectionId, HANDLE handle, int isSocket, int value, int timeout );

#endif

#endif // USE_PRIVATE

#endif
