/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL_BRIDGE_H
#define CHANNEL_BRIDGE_H

// Basisklassen
#include "../Log/DFLog.h"

// Klassen fuer Multimasterprotokolle.
#include "Channel1_2.h" // Version 1/2
#include "Channel4.h" // Version 4

#ifndef _MSC_VER
#include </usr/include/pthread.h>
#endif

struct StructBindInfo {
    int state;
    int typeDevice;
    unsigned int serial;
    char *ip[20];
};

class ChannelBridge : public CDFLog
{
public:
    // Konstruktor fuer Kanalinitialisierung
    ChannelBridge(short channelID);
    virtual ~ChannelBridge();

    // Setzt die Kanalnummer
    inline void setChannelId(int channelID) {

		// Zugewiesene Kanalnummer
		m_channelID = channelID;
		m_logCheck.m_channelID = channelID;

        // Kanalnummer setzen
        _channel4.setId(channelID);
        // Kommunikationsbasis von _channel4 an andere uebergeben
        _channel1_2.SetChannelBase(_channel4.getChannelBase());
    }

    // Setzt Klassenzustand
    inline void setActiveState(ActiveState as) { _channel4.setActiveState(as); }
    // Prueft ob Klasse bereits "in Verwendung"
    inline ActiveState getActiveState() { return _channel4.getActiveState(); }
    // Gibt direkt zurück ob der Kanal Aktiv ist
    inline bool isActive() { return (getActiveState() == is_active); }

    StructBindInfo _bindInfo;
    // Setzt die Information ob die Kanalklasse an ein bestimmtes Geraet gebunden ist.
    inline void setBind(StructBindInfo bi) { _bindInfo = bi; }
    // Prueft ob Klasse alle Informationen fuer eine Kanalbindung hat.
    inline int isBind() { return _bindInfo.state; }

    // Setzt Klassenzustand auf "Wartung"
    inline void setServiceState(int state) { _channel4.setServiceState(state); }
    // Prueft ob Klasse bereits "im Wartungsmodus"
    inline int getServiceState() { return _channel4.getServiceState(); }

    // Verbindungsannahme von uebergebenem Listen-Socket
    int connect(SOCKET *listenSocket, short channelID, short deviceID = IV_DEFAULT_DEVICEID)
    { return _channel4.connect(listenSocket, channelID, deviceID); }
    // Endesignal im Workerthread setzen und auf dessen Beendigung warten
    void directClose()
    { _channel4.directClose(); }

    // Setzen der CallBack-Funktion fuer Datensatzbenachrichtigung
    void setCallBackNotifyRecord(int (*CBNotifyRecord)(CNotifyRecord&))
    {
        _channel4.m_CBNotifyRecord = CBNotifyRecord;
    }

    // Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:

    //#################################
    // CallRoutinen fuer die Abarbeitung anstehender Kommunikationsbefehle
    //	virtual int CallFunction(short module, short func, unsigned char *data, int &count);
    // NotifyRoutine fuer die Abarbeitung von Benachrichtigungen unter den Klassen
    //	int CallFunctionNotify(short module, short func, unsigned char *data, int &count)
    //	{ return _channel4.CallFunctionNotify(module, func, data, count); }

    // Für das Projekt Spar bei Freudenberg-Id.
    // Da die DLL mit JNA eingebunden wurde ist eine mehrfachansteuerung von mehreren Threads abzufangen.
#ifdef _MSC_VER
    CRITICAL_SECTION m_mutexAPI;
#else
    pthread_mutex_t m_mutexAPI;
#endif
public:
#ifdef _MSC_VER
    DWORD _threadIdOpen;
#endif
    int tryEnterCriticalSection();
    void enterCriticalSection();
    void leaveCriticalSection();

public:
    /////////////////////////
    // Schnittstelle

    int isOpen();
    int open(int deviceID, int iType, const std::string &szComString, int iComValue, int iComTimeOut);
    int openAEIII(int iType, char *szComString, int iComValue, int iComTimeOut);
    void close(void);

    /// Ermitteln der uebergebenen Parameter an open.
    inline void getOpenParams(int &type, std::string &string, int &val, int &to) 
    { _channel4.getOpenParams((Channel4Base::enumType&)type, string, val, to); };

    // Busnummer des Kommunikationspartners
    short setDeviceIdRemote(short deviceIDRemote); 
    short getDeviceIdRemote();

    // Busnummer der DLL - Klassen
    short setDeviceId(short deviceID); 
    short getDeviceId();

    // Schnittstelle nach Fehler bereinigen
    void resetWithError(int err);

    // Speicher bereinigen
    void reset();

public:
    /////////////////////////
    // Kommunikationshelper

    int read(void *buffer, int receiveCount, int *receivedCount);
    int write(void *buffer, int transmitCount, int *transmittedCount);

    void setCallBackFunction(int (*pFunction)(void));
    int getProtocolVersion2(short deviceIDRemote, unsigned char detailed, int *piError);
    // Gibt die ermittelte Protokollversion zurueck
    unsigned char getProtocolVersion(short deviceIDRemote);
    unsigned char setProtocolVersion(short deviceIDRemote, unsigned char version);
    unsigned char _protocolVersion[251];
    bool _preferAEIII;     // Bevorzugt auf AEIII+ pruefen


    /////////////////////////
    // Zugriff auf HANDLE/SOCKET

    /// Ermitteln des Schnittstellenhandle.
    HANDLE getComPort();
    /// Setzen der Schnittstellenparameter.
    int setComPort(HANDLE hCom, int iType, int iComValue, int iTimeOut);

    int comReleaseHandle();
    int comUseHandle(HANDLE h);


    /////////////////////////
    // Helper

    // Aufbereiten und senden eines Protokollpakets
    int writePacket(short deviceIDRemote, unsigned char type, short module, short func, unsigned char *buf, int count, int transmitTrials = IV_WRITE_TRIALS);

    // Empfang und aufbereiten eines Protokollpakets
    int readPacket(short deviceID, unsigned char &type, short &module, short &func, unsigned char *buf, int &count, int receiveTrials = IV_READ_TRIALS);

    /////////////////////////
    // Befehle

    // Allgeimein

    // Anzahl der letzten Ermittlungsversuche halten.
    int _devicePollRetry;
    // Geraeteerreichbarkeit pruefen
    int checkDevice(short deviceIDRemote, int *piError, int iPollRetry);

    // PZE Master IV warmstart.
    int restart(short deviceIDRemote, int *piError);
    // Reset des Gerätes
    int reset(short deviceIDRemote, int mode, int *piError);

    // Senden einer Bestaetigungsmeldung an das AEIII+
    int sendMessage(short deviceIDRemote, unsigned char ucDelayTime,
        unsigned char ucAction,	unsigned char ucAudio, char *szMessage, 
        int iLength, int *piError);

    // Senden eines Infotextes fuer Geraetestatus.
    int sendInfotext(short deviceIDRemote, char *szMessage, int iLength, int *piError);

    // RTC - Wert schreiben / lesen
    int setDateTime(short deviceIDRemote, unsigned char *pucDateTime, int *piError);
    int getDateTime(short deviceIDRemote, unsigned char *pucDateTime, int *piError);

    // Globale-/Systemvariable schreiben / lesen
    int setGlobVar(short deviceIDRemote, unsigned char *pucVar, int iType, char *val, int *piError);
    int getGlobVar(short deviceIDRemote, unsigned char *pucVar, int iType, char *val, int iValueLength, int *piError);

    // Relais setzen/ruecksetzen und zustand abfragen.
    int closeRelay(short deviceIDRemote, int iNum, int iTime, int *piError);
    int openRelay(short deviceIDRemote, int iNum, int *piError);
    int getRelayState(short deviceIDRemote, int iNum, int *piState, int *piTime, int *piError);

    // Seriennummer auslesen
    int getSerialNum(short deviceIDRemote, int *piSerialNum, int *piError);
    // Firmwareversion auslesen
    int getVersionFirmware(short deviceIDRemote, char *pVersion, int *piError);
    // Firmwareversion aus Dateiheader lesen
    int getVersionFirmwareFromFile(char *pFileName, char *pVersion, int *piError);

    /////////////////////////
    // System Device Log
    // Datensatz lesen
    int systemReadRecord(int deviceIDRemote, char *buf, int *length, int *error);
    // Datensatz quittieren.
    int systemQuitRecord(int deviceIDRemote, int *error);
    // Datensatzzeiger ruecksetzen
    int systemRestoreRecords(short deviceIDRemote, int *piError);

    // Aktuellen Status des Flash abrufen
    int getFlashStatus(short deviceIDRemote, int *piStatus, int *piError);
public:
    /////////////////////////
    // Datensatz- / Listenbeschreibungen

    int getTableDescription(short deviceIDRemote,	unsigned char ucList, int *piError);
protected:
    int getTables4(short deviceIDRemote, char *typeName);
    int getListName(short deviceIDRemote, unsigned char ucList, char *name, int *piError);


public:
    /////////////////////////
    // Datensaetze

    // Ermitteln des ersten Datensatzes
    int collectData(short deviceIDRemote, int *piError);
    // Ermitteln des naechsten Datensatzes
    int getDatensatz(unsigned char *pucBuf, int *piLength, int *piError);
    // Pruefen ob min. ein Datensatz im AEIII+ vorhanden.
    int checkData(short deviceIDRemote, int *piError);
    // Loescht alle Datensaetzte im AEIII+
    int clearData(short deviceIDRemote, int *piError);
    // Datensatzzeiger ruecksetzen
    int restoreRecords(short deviceIDRemote, int *piError);
    // Datensatz lesen
    int readRecord(int deviceIDRemote, char *buf, int *length, int *error);
    // Datensatz quittieren.
    int quitRecord(int deviceIDRemote, int *error);
    // Anzahl der Datensaetze oder verwendetes Speichervolumen.
    int recordVolume(short deviceID,  int &number, int &memoryUsage, int &memorySize);
protected:
    // Datensatzfunktionen, Variable und Puffer fuer letzten gelesenen.
    int _recordSize, _recordError;
    unsigned char _recordData[MAX_MB_PAKET];


public:
    /////////////////////////
    // Listen

    // Laden der Importierten Listendaten
    int writeListData(short deviceIDRemote, int *piError);
protected:
    int writeListData4(short deviceIDRemote);
    //int DownloadLists(short deviceIDRemote, int *piError);

public:
    /////////////////////////
    // Zutrittslisten (Version 1 / Version 2)

    // Laden der Importierten Listendaten fuer Zutrittskontrolle
    int writeEntranceListData(short deviceIDRemote, int iListNum, int *piError);
    int writeEntrance2ListData(short deviceIDRemote, int iListNum, int *piError);
protected:
    int writeEntrance2ListData4(short deviceIDRemote, unsigned char listNum);
    int writeEntrance2List4(short deviceIDRemote, unsigned char listNum);


public:
    /////////////////////////
    // Timeboylisten

    // Laden der Importierten Listendaten fuer Timeboy
    int writeTimeboyListData(short deviceIDRemote, unsigned char id, unsigned char num, int reserve, int *piError);
protected:
    int writeTimeboyListData4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve);
    int writeTimeboyList4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve);


public:
    /////////////////////////
    // Setup

    int writeStudioFile(short deviceIDRemote, char *szFileName, int *piError);
    int readStudioFile(short deviceIDRemote, char *szFileName, int *piError);

    // Veraenderungen an der Setupdatei vornehmen
    int modifyStudioFile(char *sourceFile, char *destFile, char *changeData, int flags, int *piError);

protected:
    int writeSetupData(int *piError);
    int writeSetupData4(short deviceIDRemote);
    int writeSetupData4_2(short deviceIDRemote);
    int readSetupData(char *szFileName, int *piError);
    int readSetupData4(short deviceIDRemote);
    int readSetupData4_2(short deviceIDRemote);

public:
    /////////////////////////
    // Firmware
    int writeFileDevice(short deviceIDRemote, char *szFileName, int *piError);
protected:
    int writeDevice4(short deviceIDRemote);
    int checkFileDisplay(FileDisplay *pFileDisplay, int *piError);

public:
    /////////////////////////
    // Module
    int writeModule(short deviceIDRemote, int type, char *szFileName, int *piError);
    int writeTextData(short deviceIDRemote, char *fileName);
    int writeTouchKeyData(short deviceIDRemote, char *fileName);
    int writeDisplayColors(short deviceIDRemote, char *fileName);
    int writeWlanConfigData(short deviceIDRemote, char *fileName);
    int readWlanConfigData(short deviceIDRemote, char *fileName);
    int writeHardwareConfigData(short deviceIDRemote, char *fileName);
    int readHardwareConfigData(short deviceIDRemote, char *fileName);
    int writeFileData(short deviceIDRemote, char *fileName);
    int readFileData(short deviceIDRemote, char *fileName);

public:
    /////////////////////////
    // Log / Trace

    /// Setzen des Logdateinamen.
    void setLogFileName(const std::string &szLogFileName);

    /// Log anschalten.
    inline void setLogOn()  
    { _channel4.setTrace(TRUE); }

    /// Log ausschalten.
    inline void setLogOff() 
    { _channel4.setTrace(FALSE); }

    void setTraceFileName( const std::string &filePath )
    {
        _channel4.setTraceFilePath( filePath );
    }
    /////////////////////////
    // Passwort
    void setPassword(char *password)
    {
        _channel4.setPasswordCommunication(password);
    }
public:
    int writeMemToFile(short deviceIDRemote, int type, int address, int size, char *fileName);
    int writeFileToMem(short deviceIDRemote, char *fileName);

public:	
    /////////////////////////
    // Kanalklassen fuer Protokollverwendung.
    CChannel1_2 _channel1_2;
    Channel4 _channel4;

    inline int resultToError(int iRes, int *piError)
    {
        if (iRes < 0)
        {
            // Kommunikationsfehler
            iRes = -iRes;
            // Fehlercode umsetzen
            *piError = iRes;
            iRes = 0;
        }
        else if (iRes > 0) 
        {
            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist das Geraet zwar aktiv der Befehl
            // wird jedoch nicht unterstuetzt.
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *piError = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else if ( ((iRes >> 8) & 0xFF) == 16 ) 
            {
                // Angesprochener Slot ist nicht vorhanden.
                *piError = IV_ERROR_SLOT_NONE;
            }
            else if ( ((iRes >> 8) & 0xFF) == 17 ) 
            {
                // Kein Timeboy im Slot vorhanden.
                *piError = IV_ERROR_NO_TIMEBOY;
            }
            else if ( ((iRes >> 8) & 0xFF) == 18 ) 
            {
                // Timeboytyp noch nicht erkannt.
                *piError = IV_ERROR_TIMEBOY_NOT_READY;
            }
            else if ( ((iRes >> 8) & 0xFF) == 19 ) 
            {
                // Es wird versucht ein TimeboyI-III anzusprechen.
                *piError = IV_ERROR_TIMEBOY_I_II_III;
            }
            else if ( ((iRes >> 8) & 0xFF) == 20 ) 
            {
                // Ausführung konnte wegen belegtem Kommunikationskanal nicht weitergereicht werden.
                *piError = IV_ERROR_DOCKING_BUSY;
            }
            else if ( (iRes & 0xFF) == 28 ) 
            {
                // Passwort fuer Kommunikation wird benoetigt.
                *piError = IV_ERROR_NEED_PASSWORD;
            }
            else if ( ( iRes & 0xFF ) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *piError = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler
                *piError = IV_ERROR_PROCESSING;
            }

            // Fehler
            return 0;
        } else {

            // OK
            *piError = IV_ERROR_SUCCESS;
            iRes = 1;
        }

        return iRes;
    }
public:
    /////////////////////////
    // Schnittstelle zu Channel1/Channel2
    int getModus(short deviceIDRemote, int *error);

    /////////////////////////
    // Schnittstelle zu Channel4
    void setCallBackFunctionWithParams(int (*pFunction)(unsigned int, unsigned int));
    unsigned int crc32(unsigned char *data, unsigned int size, unsigned int crc = 0xFFFFFFFF, unsigned char closure = TRUE);

    // MODULE_HARDWARE

    // Einfache Anfrage ob das Geraet antwortet.
    int ping(short deviceIDRemote);

    // Typ lesen
    int setTypeDevice(unsigned char deviceID, unsigned char type);
    int getTypeDevice(short deviceIDRemote, unsigned char &type);
    int setTypeBoard(unsigned char deviceID, unsigned char type);
    int getTypeBoard(short deviceIDRemote, unsigned char &type);
    int setTypeTCPIP(unsigned char deviceID, unsigned char val);
    int getTypeTCPIP(short deviceIDRemote, unsigned char &val);
    int setTypeModemMC(unsigned char deviceID, unsigned char val);
    int getTypeModemMC(short deviceIDRemote, unsigned char &val);
    int setTypeDisplay(unsigned char deviceID, unsigned char type);
    int getTypeDisplay(short deviceIDRemote, unsigned char &type);
    int setTypeKeypad(unsigned char deviceID, unsigned char type);
    int getTypeKeypad(short deviceIDRemote, unsigned char &type);
    int setTypeTransponder(unsigned char deviceID, unsigned char type);
    int getTypeTransponder(short deviceIDRemote, unsigned char &type);
    int setTypeMMC(unsigned char deviceID, unsigned char type);
    int getTypeMMC(short deviceIDRemote, unsigned char &type);
    int setTypeRadio(unsigned char deviceID, unsigned char type);
    int getTypeRadio(short deviceIDRemote, unsigned char &type);
    int setTypeGPS(unsigned char deviceID, unsigned char val);
    int getTypeGPS(short deviceIDRemote, unsigned char &type);
    int setTypeDCF77(unsigned char deviceID, unsigned char type);
    int getTypeDCF77(short deviceIDRemote, unsigned char &type);
    int setTypeFingerprint(unsigned char deviceID, unsigned char type);
    int getTypeFingerprint(short deviceIDRemote, unsigned char &type);
    int setTypeExternalReader(unsigned char deviceIDRemote, unsigned char val);
    int getTypeExternalReader(short deviceIDRemote, unsigned char &val);
    int setTypeTouch(unsigned char deviceID, unsigned char val);
    int getTypeTouch(short deviceID, unsigned char &val);
    int setTypeSensor(unsigned char deviceID, unsigned char val);
    int getTypeSensor(short deviceID, unsigned char &val);

    // Komponente lesen
    int setComponentRS485_0(unsigned char deviceID, unsigned char val);
    int getComponentRS485_0(short deviceIDRemote, unsigned char &val);
    int setComponentRS485_1(unsigned char deviceID, unsigned char val);
    int getComponentRS485_1(short deviceIDRemote, unsigned char &val);
    int setComponentUSB(unsigned char deviceID, unsigned char val);
    int getComponentUSB(short deviceIDRemote, unsigned char &val);
    int setComponentInAnalog(unsigned char deviceID, unsigned int val);
    int getComponentInAnalog(short deviceIDRemote, unsigned int &val);
    int setComponentInDigital(unsigned char deviceID, unsigned int val);
    int getComponentInDigital(short deviceIDRemote, unsigned int &val);
    int setComponentOutDigital(unsigned char deviceID, unsigned int val);
    int getComponentOutDigital(short deviceIDRemote, unsigned int &val);
    int setComponentBattery(unsigned char deviceID, unsigned char val);
    int getComponentBattery(short deviceID, unsigned char &val);
    int setComponentCharge(unsigned char deviceID, unsigned char val);
    int getComponentCharge(short deviceID, unsigned char &val);

    // Option schreiben/lesen
    int setOptionIOModule(unsigned char deviceID, unsigned char val);
    int getOptionIOModule(short deviceIDRemote, unsigned char &val);
    int setOptionTBAdaption(unsigned char deviceID, unsigned char val);
    int getOptionTBAdaption(short deviceIDRemote, unsigned char &val);
    int setOptionComServer(unsigned char deviceID, unsigned char val);
    int getOptionComServer(short deviceIDRemote, unsigned char &val);
    int setOptionTPWrite(unsigned char deviceID, unsigned char val);
    int getOptionTPWrite(short deviceIDRemote, unsigned char &val);
    int setOptionHTTP(unsigned char deviceID, unsigned char val);
    int getOptionHTTP(short deviceIDRemote, unsigned char &val);
    int setOptionLanguage(unsigned char deviceID, unsigned char val);
    int getOptionLanguage(short deviceIDRemote, unsigned char &val);
    int setOptionBiokeyUpdate(unsigned char deviceID, unsigned char val);
    int getOptionBiokeyUpdate(short deviceIDRemote, unsigned char &val);
    int setSetupOptions(short deviceIDRemote, unsigned short mask, unsigned short options);
    int getSetupOptions(short deviceIDRemote, unsigned short &mask, unsigned short &options);
    int setOptionFirmware(short deviceIDRemote, int mask, int options);
    int getOptionFirmware(short deviceIDRemote, int &mask, int &options);
    int setPayableOptions(unsigned char deviceIDRemote, unsigned char mask, unsigned char options);
    int getPayableOptions(short deviceIDRemote, unsigned char &mask, unsigned char &options);



    // HWConfig lesen
    int setHWConfig(unsigned char deviceID, unsigned char val);
    int getHWConfig(short deviceIDRemote, unsigned char &val);

    // Gesamte Hardwareinfo lesen
    int getHardwareInfo(short deviceIDRemote, StructHardwareInfo &hwInfo);
    // Gesamte Hardwareinfo lesen
    int getHardwareInfo( short deviceIDRemote, HardwareInfo &hardwareInfo );
    // Gesamte Hardwareinfo in String umsetzen.
    int getInfos(StructHardwareInfo hwInfo, string &info);


    // MODULE_SYSTEM

    // Systemreset; Speicherbereiche loeschen
    //	int SystemReset(short deviceIDRemote, unsigned short modus);

    // Firmwareversion lesen
    int getVersionFirmware(short deviceIDRemote, char *version);
    int getVersionBootloader(short deviceIDRemote, char *version);

    // Seriennummer 
    int setSerialNumber(unsigned char deviceID, unsigned int serial);
    int getSerialNumber(short deviceIDRemote, unsigned int &serial);

    // Datum/Uhrzeit setzen/lesen
    //	int SetRTC(short deviceIDRemote, char *time);
    //	int GetRTC(short deviceIDRemote, char *time);

    // Systemvariable setzen/lesen
    //	int SetSystemVariable(short deviceIDRemote, char *name, char *val);
    //	int GetSystemVariable(short deviceIDRemote, char *name, char *val);

    // Relais setzen/lesen
    //	int SetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time);
    //	int GetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time);

    // Datensegmentuebertragung beginnen/enden
    int beginSegment(short deviceIDRemote, unsigned short art, unsigned int &address, unsigned int size, unsigned int crc);
    int endSegment(short deviceIDRemote, unsigned short art, unsigned int crc);

    // Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
    int setFlash(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);
    int getFlash(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);
    int getFlashPage(short deviceIDRemote, unsigned short pageNum, unsigned short offs, unsigned char *buf, int &count);

    // EEPROM setzen/lesen
    int setEEPROM(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);
    int getEEPROM(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);

    // RAM setzen/lesen
    int setRAM(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);
    int getRAM(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);

    // PROGRAM setzen/(lesen unterbunden)
    //	int SetProgram(short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);
    //	int GetProgram(short deviceIDRemote, unsigned int address, unsigned char *buf, unsigned short count);

    // MMC setzen/lesen 
    int setMMC(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);
    int getMMC(short deviceIDRemote, unsigned int address, unsigned char *buf, int count);

    // Systembios aufrufen/verlassen
    int enterBios(short deviceIDRemote, unsigned char protocolVersion);
    int exitBios(short deviceIDRemote);

    // Interface setzen/lesen (RS232, TCPIP...)
    int setInterface(short deviceIDRemote, unsigned char type);
    int getInterface(short deviceIDRemote, unsigned char &type);

    // Baud-Rate setzen/lesen
    int setBaud(short deviceIDRemote, unsigned int br);
    int getBaud(short deviceIDRemote, unsigned int &br);

    // DeviceID setzen/lesen
    int setDeviceIdBios(short deviceIDRemote, unsigned short id);
    int getDeviceIdBios(short deviceIDRemote, unsigned short &id);

    // Bootloader aktivieren
    int enterBootloader(short deviceIDRemote, unsigned char protocolVersion);

    // Monitor Mode vom XPort
    int enterXPortMonitor(short deviceIDRemote, char *data, int &count);
    int enterXPortMonitorExt(short deviceIDRemote, char *data, int &count);
    int commandXPort(short deviceIDRemote, char *buf, int &count);
    int exitXPortMonitor(short deviceIDRemote);

    // Oeffnen einer Tabelle
    int tableOpen(short deviceIDRemote, char *name, int options, int &handle, char *reserved);
    // Schliessen der Tabelle
    int tableClose(short deviceIDRemote, int handle);

    // Setzen eines Filters auf die Tabelle
    int tableSetFilter(short deviceIDRemote, int handle, char *filter);
    // Lesen des Filters auf die Tabelle
    int tableGetFilter(short deviceIDRemote, int handle, char *filter, short &len);
    // Loeschen des Filters der Tabelle
    int tableClearFilter(short deviceIDRemote, int handle);

    // Ueberspringen von n Datensaetzen (offset kann negativ sein), ab top/aktuell/bottom
    int tableSkip(short deviceIDRemote, int handle, int offset, short origin);

    // Aendern eines Spaltenwerts, des aktuellen Datensatzes
    int tableSetField(short deviceIDRemote, int handle, char *name, char *val);
    // Abfragen eines Spaltenwerts, des aktuellen Datensatzes
    int tableGetField(short deviceIDRemote, int handle, char *name, char *val, short &len);

    // Abfragen einer Modulinformation
    int getInfo(short deviceIDRemote, char *name, char *params, char *info, int &len);

    // Zutrittsbuchung durchfuehren
    int entrance2Identification(short deviceIDRemote, char *TM, char *Id, char *Pin, char *status, int &statusLength);
    // LED, Relais auf Tuermodul schalten
    int entrance2OnlineAction(short deviceID, char *TM, unsigned int mask, unsigned char type, unsigned char duration);

    // Transparentmodus aktivieren/deaktivieren
    int openComServerMode(short deviceIDRemote, char *comm, unsigned int baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose);
    int closeComServerMode(short deviceIDRemote);

    // Aktive Meldungen aktivieren/deaktieren
    int setNotifyOnDevice(short deviceIDRemote, int type, int state);

    // Softwareinformation der Anwendungsfirmware abfragen
    int getSoftwareInfo(short deviceIDRemote, unsigned char *swi, int &infoLength);

    // Passwort pruefen
    int passwordCheck(short deviceID, char *password);
    // Passwortschlüssel abrufen
    int getPasswordKey(short deviceID, char *key);

    // Virtuellen Tastendruck ausführen.
    int pressVirtualKey(short deviceIDRemote, unsigned char vk, unsigned char flags, int *piError);

    // MODULE_TABLE
#define BIOKEY_OK                                (0x80)  ///<letzer Befehl erfolgreich
#define BIOKEY_ERROR                             (0x81)  ///<unerwarteter Fehler aufgetreten
#define BIOKEY_BAD_IMAGE                         (0x82)  ///<kein oder schlechtes Bild
#define BIOKEY_BAD_FPT                           (0x83)  ///<schlechtes Template, nicht genug Merkmale
#define BIOKEY_DB_FULL                           (0x84)  ///<Datenbank voll
#define BIOKEY_DB_EMPTY                          (0x85)  ///<Datenbank leer
#define BIOKEY_IDS_USED                          (0x86)  ///<übergebene PIDs werden schon benutzt
#define BIOKEY_IDS_UNKNOWN                       (0x87)  ///<übergebene PIDs existieren nicht
#define BIOKEY_RECOGNIZED                        (0x88)  ///<Person wurde erkannt
#define BIOKEY_NOT_RECOGNIZED                    (0x89)  ///<Person wurde nicht erkannt
#define BIOKEY_NO_IMAGE                          (0x8A)  ///<es wurde kein Bild vom Sensor aufgenommen
#define BIOKEY_NO_MATCH                          (0x8B)  ///<kein Treffer
#define BIOKEY_AUTH_FAILED                       (0x8C)  ///<falsches Passwort
#define BIOKEY_FPT_EXISTS                        (0x8D)  ///< an attempt to enroll with message cmd 0x26 failed because a template that matches the newly acquired one was found in the database
#define BIOKEY_FPT_ENROLLED                      (0x8E)  ///< a newly acquired fingerprint has been enrolled successfully
#define BIOKEY_TIMEOUT                           (0x8F)  ///< a timeout occurred during an enroll/identify process
#define BIOKEY_FPT_READ                          (0x90)  ///< a fingerprint was read during “Enroll best of three” procedure
    // Datensatz an Fingerprint-Tabelle anfuegen.
    int fingerprintAppendRecord(short deviceIDRemote, unsigned char type, unsigned char *fingertemplate, int size);
    // Datensatz aus Fingerprint-Tabelle lesen.
    int fingerprintGetRecord(short deviceIDRemote, unsigned char &type, int pid, int fid, unsigned char *fingertemplate, int &size);
    // Datensatz aus Fingerprint-Tabelle loeschen.
    int fingerprintDeleteRecord(short deviceIDremote, int pid, int fid);
    // Tabelle mit Fingerprintdaten erzeugen
    int fingerprintList(short deviceIDRemote, int flags, int &recordCount);
    // Backup der Fingerprints erstellen
    int fingerprintBackup(short deviceIDRemote, char *backupFile, int flags);
    // Wiederherstellen der Fingerprints
    int fingerprintRestore(short deviceIDRemote, char *backupFile, int flags);

    // Systemflags setzen/lesen
    int setSystemFlags(short deviceIDRemote, unsigned int flags);
    int getSystemFlags(short deviceIDRemote, unsigned int &flags);

    // MODULE_APPLICATION
    // Setzt den Font für die Anzeige der Message und Info - Text
    int setTypeFont(short deviceIDRemote, int iType, int *piError);

    // Setzt das Passwort fuer die verschluesselte Kommunikation
    int setCommunicationPassword(unsigned char *password, int size, int sessionTimeout, int global);
    int isGlobalCommunicationPassword();

private:
    int _protocolVersionRecordDescription;
    int _protocolVersionListDescription;

public:
    // Funktionen für Pufferzugriff.
    int GetTableCount(unsigned char ucList = 0);

    int GetTableRecord(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount, unsigned char *pucVariable = NULL, 
        unsigned char ucList = 0);

    int GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength, unsigned char ucList = 0);

    void ClearListBuffer();
    int MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);

    void ClearEntranceListBuffer(int iListNum = -1);
    int MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);

    void ClearEntrance2ListBuffer(int iListNum = -1);
    int MakeEntrance2List(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);

    void ClearTimeboyListBuffer(unsigned char id = 0xFF, unsigned char num = 0xFF);
    int MakeTimeboyList(unsigned char id, unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned int size, int flags, int *error);

    bool WriteListToBinFile( unsigned char listNum, const char *filePath );
    bool WriteListToTextFile( unsigned char listNum, const char *filePath );
    bool WriteListEntrance2ToBinFile( unsigned char listNum, const char *filePath );
    bool WriteListEntrance2ToTextFile( unsigned char listNum, const char *filePath );
    bool WriteListTimeboyToBinFile( unsigned char listId, unsigned char listNum, const char *filePath );

    std::string GetFileDeviceFileName();
    std::string GetFileDeviceError();
    std::string GetFileDeviceInfo();
    FileDevice &GetFileDevice();

    // Zutrittsbuchung durchfuehren
    int accessControlIdentification(short deviceIDRemote, int masterId, char *TM, char *Id, char *Pin, char *status, int &statusLength);
    // LED, Relais auf Tuermodul schalten
    int accessControlOnlineAction(short deviceID, int masterId, char *TM, unsigned int mask, unsigned char type, unsigned char duration);
    int accessControlKnobCommand( short deviceIDRemote, unsigned char knobCommand, unsigned short masterId, unsigned char knobId, char *params, unsigned char *data, int *sizeData );

	int customCommand( short deviceIDRemote, unsigned int idendification, unsigned int customCommand, char *data, int *sizeData );

	int startTransparencyMode( unsigned char deviceAddress, unsigned short id, unsigned short flags, unsigned int sizeReadBuffer, unsigned int sizeWriteBuffer, int timeout, int *errorNumber );
    int stopTransparencyMode( unsigned char deviceAddress, int *errorNumber );
    int isTransparencyMode();

    int ping( unsigned char deviceAddress, int attempts, int *errorNumber );
};

#endif // !defined(AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_)

