/**
* @file  FileHardwareInfo.cpp
* @brief Implementierung der Klasse FileHardwareInfo.
*/
//////////////////////////////////////////////////////////////////////

#include "filehardwareinfo.h"
#include <memory.h>
#include <stdio.h>

#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <boost/format.hpp>


/*! Schnittstelle zwischen der Datei mit den Daten für die HardwareConfig und den Softwaremodulen.
    Die Daten werden wegen ihrer Baumartigen Art nach dem Laden der Datei in eine Baumstruktur von
    ItemHardwareInfo Einträgen gewandelt. Änderungen bitte in diesem Item-Baum vornehmen, über 
    rootItemHardwareConfig() erhält man zugriff auf das Wurzelitem.
    Die Baumstruktur wird vor dem Speichern serialisiert und in der Datei gespeichert.
*/
FileHardwareInfo::FileHardwareInfo()
{
    // Reset der sonstigen Umgebung durchführen.
    reset();
}

FileHardwareInfo::FileHardwareInfo( const FileHardwareInfo &hwInfo )
{
    // Klasse initialisieren.
	reset();

	_filePath = hwInfo._filePath;
	_crc = hwInfo._crc;
	_buffer = hwInfo._buffer;

	// Member durch gelesene Daten initialisieren und pruefen
	if ( _buffer.size() >= (sizeof(HARDWARECONFIGNODE) + sizeof(unsigned int) + sizeof(HARDWARECONFIG)) )
	{
		// Pruefsumme zum Vergleich berechnen.
		if ( _crc != GetCRC32( _buffer.data(), _buffer.size() ) )
		{
			// Angegebene Prüfsumme stimmt nicht mit der berechneten überein.
		}

		HARDWARECONFIGNODE *hcn = (HARDWARECONFIGNODE*)&_buffer.front();
		if ( hcn->version != 1 )
		{
			// Version der Dateidaten aktueller als erwartet.
		}

		// Aus den Serialisierungsdaten die Baumrepresentation erstellen.
		if ( !readSerialize() )
		{
			// Erstellen der Baumdaten gescheitert.
		}
	}
	else
	{
		// Keine Daten vorhanden
	}
}

FileHardwareInfo::~FileHardwareInfo()
{
}

FileHardwareInfo &FileHardwareInfo::operator=( const FileHardwareInfo &hwInfo )
{
    // Klasse initalisieren
	reset();

	_filePath = hwInfo._filePath;
	_crc = hwInfo._crc;
	_buffer = hwInfo._buffer;

	// Member durch gelesene Daten initialisieren und pruefen
	if ( _buffer.size() >= (sizeof(HARDWARECONFIGNODE) + sizeof(unsigned int) + sizeof(HARDWARECONFIG)) )
	{
		// Pruefsumme zum Vergleich berechnen.
		if ( _crc != GetCRC32( _buffer.data(), _buffer.size() ) )
		{
			// Angegebene Prüfsumme stimmt nicht mit der berechneten überein.
		}

		HARDWARECONFIGNODE *hcn = (HARDWARECONFIGNODE*)&_buffer.front();
		if ( hcn->version != 1 )
		{
			// Version der Dateidaten aktueller als erwartet.
		}

        // Aus den Serialisierungsdaten die Baumrepresentation erstellen.
        if ( !readSerialize() )
        {
            // Erstellen der Baumdaten gescheitert.
        }
	}
	else
	{
		// Keine Daten vorhanden
	}

	return *this;
}

/*! Initalisiert die Klassendaten.
    Nach dem Aufruf stellt die Klasse eine Leere Datei dar.
*/
void FileHardwareInfo::reset()
{
    // Basis Reset
    FileIntelHex::reset();

    // Member initialisieren

    // Dateiname
    _filePath.clear();

    // Pruefsumme zum Test auf Datenaenderung ruecksetzen.
    _crc = 0;

    // Signature Call-Back
    signing = 0;

    // Der Standardmäßig zu verarbeitende Datensatztyp
    setRecordType( TypeHardwareConfigData );

    // Vorhandene Konfiguration löschen.
    _deviceHardwareInfo.reset();
}

/*! Laden einer HardareConfig-Datei und wandeln der Daten in eine interne 
    Baumdarstellung von ItemHardwareInfo-Klassen.
    Die Datei muss in einem Intel-Hex-Format vorliegen. Die Endung sollte *.hex oder *.df1 sein.
*/
bool FileHardwareInfo::load( const std::string &filePath ) 
{
    // Daten durch die Basisklasse in _buffer laden.
    if ( !FileIntelHex::load( filePath ) )
        return false;

    // Member durch gelesene Daten initialisieren und pruefen
    if ( _buffer.size() >= (sizeof(HARDWARECONFIGNODE) + sizeof(unsigned int) + sizeof(HARDWARECONFIG)) )
    {
        // Pruefsumme zum Test auf Datenaenderung berechnen.
        _crc = GetCRC32( _buffer.data(), _buffer.size() );

        HARDWARECONFIGNODE *hcn = (HARDWARECONFIGNODE*)&_buffer.front();
        if ( hcn->version != 1 )
        {
            // Version der Dateidaten aktueller als erwartet.
			setErrorState( Version );
			setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
            return false;
        }

        // Aus den Serialisierungsdaten die Baumrepresentation erstellen.
        return readSerialize();
    }
    else
    {
		// Fehler zuweisen
		setErrorState( NoData );
		setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
    }

    return false;
}

/*!
    Hilfsfunktion zum manuellen Serialisieren der in _memHardwareConfig enthaltenen Daten.
*/
bool FileHardwareInfo::readSerialize()
{
    // Aus den Serialisierungsdaten die Baumrepresentation erstellen.
    if ( !_deviceHardwareInfo.readSerialize( _buffer ) )
    {
        // Erstellen der Dateidaten gescheitert
        setErrorState( SerializeFileData );
        setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
        return false;
    }

    return true;
}

/*! Speichern der internen Darstellung von ItemHardwareInfo-Klassen in eine HardwareConfig-Datei.
    Die Datei wird im Intel-Hex-Format gespeichert. Die Endung sollte *.hex oder *.df1 sein.
*/
bool FileHardwareInfo::save( const std::string &filePath, bool append ) 
{
    // Serialisierung der Baumstruktur in das zu schreibende Array durchfuehren.
    if ( !writeSerialize() )
        return false;

    // Signatur ggf. nachtragen.
    if ( signing )
    {
        HARDWARECONFIGNODE *hcn = (HARDWARECONFIGNODE*)&_buffer.front();
        hcn->signature = signing( &_buffer[hcn->ptrHardwareConfigArray], _buffer.size() - hcn->ptrHardwareConfigArray );
        hcn->signatureSize = _buffer.size() - hcn->ptrHardwareConfigArray;
    }

    // Nur bei einer Dateiangabe auch wirklich in diese schreiben.
    if ( filePath.size() )
    {
        // Daten der HARDWARE-Konfiguration sichern.
        if ( !FileIntelHex::save( filePath, append ) )
            return false;
    }

    // Pruefsumme zum Test auf Datenaenderung berechnen.
    _crc = GetCRC32( _buffer.data(), _buffer.size() );

    return true;
}

/*!
    Hilfsfunktion zum manuellen Serialisieren der in _memHardwareConfig enthaltenen Daten.
*/
bool FileHardwareInfo::writeSerialize()
{
    // Aktuellen Speicher leeren.
    _buffer.clear();
    // Serialisierung der Baumstruktur in das zu schreibende Array durchfuehren.
    if ( !_deviceHardwareInfo.writeSerialize( _buffer ) )
    {
        // Erstellen der Dateidaten gescheitert
        setErrorState( SerializeFileData );
        setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
        return false;
    }

    return true;
}

setItemBoard FileHardwareInfo::collectModuleIds( const setModuleId &ids )
{
    return rootItemDeviceHardwareInfo().collectModuleIds( ids );
}

setItemBoard FileHardwareInfo::moduleRS232Communication( const setModuleRS232Communication &module )
{
    return rootItemDeviceHardwareInfo().moduleRS232Communication( module );
}

setItemBoard FileHardwareInfo::moduleRS485Communication( const setModuleRS485Communication &module )
{
    return rootItemDeviceHardwareInfo().moduleRS485Communication( module );
}

setItemBoard FileHardwareInfo::moduleUSBCommunication( const setModuleUSBCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleUSBCommunication( module );
}

setItemBoard FileHardwareInfo::moduleLANCommunication( const setModuleLANCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleLANCommunication( module );
}

setItemBoard FileHardwareInfo::moduleWLANCommunication( const setModuleWLANCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleWLANCommunication( module );
}

setItemBoard FileHardwareInfo::moduleMobileCommunication( const setModuleMobileCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleMobileCommunication( module );
}

setItemBoard FileHardwareInfo::moduleBluetoothCommunication( const setModuleBluetoothCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleBluetoothCommunication( module );
}

setItemBoard FileHardwareInfo::moduleMMCCommunication( const setModuleMMCCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleMMCCommunication( module );
}

setItemBoard FileHardwareInfo::moduleUSBHostCommunication( const setModuleUSBHostCommunication &module )
{
    return rootItemDeviceHardwareInfo().moduleUSBHostCommunication( module );
}

setItemBoard FileHardwareInfo::moduleAccessControl( const setModuleAccessControl &module )
{
    return rootItemDeviceHardwareInfo().moduleAccessControl( module );
}

setItemBoard FileHardwareInfo::moduleBiokey( const setModuleBiokey &module )
{
    return rootItemDeviceHardwareInfo().moduleBiokey( module );
}

setItemBoard FileHardwareInfo::moduleGPS( const setModuleGPS &module )
{
    return rootItemDeviceHardwareInfo().moduleGPS( module );
}

/*! Stellt die Automatische Korrektur von Fehlenden Steckmodulen ein während dem Laden der Daten ein.

*/
void FileHardwareInfo::setAutoCorrectModulesDisabled( bool state )
{
    rootItemDeviceHardwareInfo().setAutoCorrectModuleDisabled( state );
}