// FileStudio.cpp: Implementierung der Klasse FileStudio.
//
//////////////////////////////////////////////////////////////////////

#include "filestudio.h"
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <fstream>

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

FileStudio::FileStudio()
{
    // Speicher fuer EEP reservieren
    m_eep = new unsigned char[EEP_SIZE_SU];

    // Speicher fuer Dateiheader reservieren
    m_header = new unsigned char[EEP_SIZE - EEP_SIZE_SU];

    // Speicher fuer Flash reservieren
    m_flash = new unsigned char[FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU + 100];

    // Initialisieren der 32bit-CRC Umgebung.
    InitCRC32Table();

    // Dokument aufsetzen
    SetDefault();
}

FileStudio::~FileStudio()
{
    delete[] m_eep;
    delete[] m_header;
    delete[] m_flash;
}

void FileStudio::SetDefault( int flags )
{
    // Eeprom 
    memset( m_eep, 0xFF, EEP_SIZE_SU );

    // Header auf Standard setzen.
    SetDefaultHeader();

    // Zusaetlicher Setupspeicher
    m_flashSizeSU2 = FLASH_SIZE_SU2;
    // Flashbereiche
    memset( m_flash, 0xFF, FLASH_SIZE_SU + FLASH_SIZE_SU2 );

    // Konfigbereich
    memset( &m_config, 0, sizeof(StructConfig) );
    m_configMem = (unsigned char*)&m_config;

    // Extended Flash
    m_flashExtended = &m_flash[FLASH_SIZE_SU + FLASH_SIZE_SU2];
    memset( m_flashExtended, 0, EXTENDED_SIZE_SU + 100 );

    // Der erweiterte Speicherbereich wird mit einer dynamischen Struktur begonnen,
    // deren erster Parameter ein Check - UINT32 - Wert enthaelt.
    StructExtendedFlash extBegin;
    // Alles mit Null initialisieren.
    memset( &extBegin, 0, sizeof(StructExtendedFlash) );
    // Struktur fuer erweiterten Bereich aufsetzen
    extBegin.check = EXTENDED_CHECK;
    extBegin.size  = sizeof(StructExtendedFlash);
    extBegin.offsetFreeScope = extBegin.size;
    // Flag fuer Verarbeitung
    extBegin.flags = EXTF_NEW;

    *(StructExtendedFlash*)m_flashExtended = extBegin;

    // Die Listenangaben werden bei gesetzten flag 1 beibehalten?
    if ( (flags & 0x01) != 0x01 )
    {
        // Listenpfade
        memset( m_listFileName, 0, sizeof(m_listFileName) );
        memset( m_listEntranceFileName, 0, sizeof(m_listEntranceFileName) );
        memset( m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName) );
        memset( m_strExtendedFilePath, 0, sizeof(m_strExtendedFilePath) );
    }
}

void FileStudio::SetDefaultHeader()
{
    // Dateiheader 
    memset( m_header, 0, EEP_SIZE - EEP_SIZE_SU );
    m_fileHeader = (StructFileStudioHeader*)m_header;
    // Dateikopf aufsetzen
    m_fileHeader->check = FILESTUDIOHEADER_CHECK;
    m_fileHeader->bugfixSize = sizeof_StructFileStudioHeader_1;
    m_fileHeader->size = sizeof(StructFileStudioHeader);

    // Es wird mindestens diese Version benoetigt.
    // Die Version wird ggf. auf die benoetigte in der jeweiligen MakeImage-Funktion angepasst.
    NeedStudioVersion( NEED_STUDIO_VERSION );
    NeedLibraryVersion( NEED_LIBRARY_VERSION );
}

/*
return:
0: Fehler beim Erstellen der Headerdaten eingetreten.
1: Dateiheaderdaten erfolgreich erstellt und eingetragen.
*/
int FileStudio::buildHeader()
{
    // CRC der Dateibereiche
    m_fileHeader->crcEep = GetCrcEep();
    m_fileHeader->crcEepExtended = 0; // War in Version 1, CRC von Restheaderbereich berechnen und eintragen
    m_fileHeader->crcFlash = GetCrcFlash();
    m_fileHeader->crcListFile = GetCrcListFile();
    m_fileHeader->crcConfig = GetCrcConfig();
    m_fileHeader->crcFlashExtended = GetCrcFlashExtended();
    m_fileHeader->crcListEntranceFile = GetCrcListEntranceFile();
    m_fileHeader->crcListEntrance2File = GetCrcListEntrance2File();
    m_fileHeader->crcListExtendedFile = GetCrcListExtendedFile();
    
    // Size und CRC der Ersten Version immer eintragen.
    m_fileHeader->bugfixSize = sizeof_StructFileStudioHeader_1;
    m_fileHeader->bugfixCRC = GetCRC32( m_header, m_fileHeader->bugfixSize - sizeof(unsigned int) );
    
    // Size dieser Version eintragen, wenn noch nicht aktuell.
    if ( m_fileHeader->size < sizeof(StructFileStudioHeader) )
        m_fileHeader->size = sizeof(StructFileStudioHeader);

    // Gesamt-CRC fuer Header berechnen und eintragen.
    *(unsigned int*)&m_header[m_fileHeader->size - sizeof(unsigned int)] = GetCRC32( m_header, m_fileHeader->size - sizeof(unsigned int) );

    // Wenn eine aktueller Version als hier aktuell vorhanden geladen wurde:
    // Die Groessenangabe und Daten werden beibehalten.
    // Es kann davon ausgegangen werden, dass im vorangegangenen Ladeverlauf die Mindestversion des Studio oder Library geprueft wurde.
    // Wenn das Laden erfolgreich durchgefuehrt werden konnte handelt es sich um hinzugefuegte Daten die nicht so relevant sind oder ggf. gar nicht verwendet werden.
    // Es wird in einem solchen Fall mit der Struktur gearbeitet wie sie ist.

    return 1;
}

/*
return:
0: Laengenangabe oder Pruefsumme des Headers falsch.
1: Pruefungen erfolgreich und ohne Fehler durchgefuehrt.
*/
int FileStudio::checkHeader()
{
    if ( IsHeader() )
    {
        // Version 1, Angaben sind bei allen Versionen verfuegbar.
        // Laengenangabe liegt im moeglichen Bereich?
        if ( m_fileHeader->bugfixSize != sizeof_StructFileStudioHeader_1 )
        {
            return 0;
        }

        // Pruefen ob die Headerdaten ggf. korrupt sind?
        if ( *(unsigned int*)&m_header[m_fileHeader->bugfixSize - sizeof(unsigned int)] != GetCRC32( m_header, m_fileHeader->bugfixSize - sizeof(unsigned int) ) )
            return 0;


        // Version >= 2
        // Laengenangabe liegt im moeglichen Bereich?
        if ( m_fileHeader->size != 0 && (m_fileHeader->size < sizeof_StructFileStudioHeader_2 || m_fileHeader->size > (EEP_SIZE - EEP_SIZE_SU)) )
        {
            return 0;
        }
        else if ( m_fileHeader->size != 0 )
        {
            // Pruefen ob die Headerdaten ggf. korrupt sind?
            if ( *(unsigned int*)&m_header[m_fileHeader->size - sizeof(unsigned int)] != GetCRC32( m_header, m_fileHeader->size - sizeof(unsigned int) ) )
                return 0;

            // CRC loeschen, da geprueft und sich ggf. mit Werten dieser Stukturversion ueberlagert.
            *(unsigned int*)&m_header[m_fileHeader->size - sizeof(unsigned int)] = 0;
        }
        else // size == 0
        {
            // Dahinterliegende Daten als ungueltig behandeln.
            memset( &m_header[m_fileHeader->bugfixSize], 0, (EEP_SIZE - EEP_SIZE_SU) -  m_fileHeader->bugfixSize );
        }

        // Version 1
        // CRC loeschen, da geprueft.
        *(unsigned int*)&m_header[m_fileHeader->bugfixSize - sizeof(unsigned int)] = 0;

        // Ist Version 2?
        //if ( m_fileHeader->size == sizeof_StructFileStudioHeader_2 )

        // Wenn eine aktueller Version als hier aktuell vorhanden geladen wuerde:
        // Es kann davon ausgegangen werden, dass im nachfolgenden Ladeverlauf die Mindestversion des Studio oder Library nicht erreicht wird,
        // wenn nicht, dann handelt es sich um hinzugefuegte Daten die nicht so relevant sind oder ggf. gar nicht verwendet werden.
        // Es wird in einem solchen Fall mit der Struktur geargeitet wie sie ist.
    }

    return 1;
}

/*
return:
1: Laden OK
0: Laden ERROR
Param error bei Laden ERROR
-1:
-2: Öffnen der Datei gescheitert.
-3: Lesen aus der Datei gescheitert.
-4: Prüfsummenfehler im Dateiinformationblock.
-5: Zum Öffnen der Setupdatei wird mindestens das DatafoxStudioIV - Version m_file.m_fileHeader->minStudioVersion benötigt.
    In der Setupdatei werden Einstellungen und Erweiterungen verwendet, die durch dieses DatafoxStudioIV noch nicht unterstützt werden. Bitte aktualisiseren Sie die Anwendung auf die angegebene Mindestversion.
-6: Warnung: Um alle in der Setupdatei vorliegenden Informationen zu Unterstützen wird mindestens das DatafoxStudioIV - Version m_file.m_fileHeader->shouldStudioVersion benötigt.
    Ein Abspeichern der Setupdatei mit dem vorliegenden DatafoxStudioIV führt zu einem Verlust von Verwaltungsspezifischen Dateidaten.
-7: Zum Öffnen der Setupdatei wird mindestens die DFComDLL - Version m_file.m_fileHeader->minLibraryVersion benötigt.
    In der Setupdatei werden Einstellungen und Erweiterungen verwendet, die durch diese DFComDLL noch nicht unterstützt werden. Bitte aktualisiseren Sie die Bibliothek auf die angegebene Mindestversion.
-8: Warnung: Um alle in der Setupdatei vorliegenden Informationen zu Unterstützen wird mindestens die DFComDLL - Version m_file.m_fileHeader->shouldLibraryVersion benötigt.
    Ein Abspeichern der Setupdatei mit dem vorliegenden DatafoxStudioIV führt zu einem Verlust von Verwaltungsspezifischen Dateidaten.
-9: Unerwartetes Setupdateiformat.
-10: 
-11: 
-12: 
-13: 
-14: 
-15: 
-16: 
-17: 
-18: Prüfsummenfehler im Dateninhalt.
*/
int FileStudio::Load( const char *fileName, int *error, int flags )
{
    int res = 1;
    struct stat st;
    char *fName;
    char mTimeStamp[20]; // 2009-05-19 09:16:22
    unsigned char buf[2]; // Laengenangabe bei Strings verarbeiten
    size_t length;
    int i;
    int listFileSize = 0;
    int entranceListFileSize = 0;
    int entrance2ListFileSize = 0;
    int extendedFilePathSize = 0;
    size_t maxHeaderSize = EEP_SIZE - EEP_SIZE_SU;
    unsigned int sizeSetup;
    struct tm *ptm;
    StructExtendedFlash extBegin;

    if ( stat( fileName, &st ) != 0 )
    {
        *error = -1;
        return 0;
    }

    FILE *fp = fopen( fileName, "rb" );
    if ( fp == NULL )
    {
        *error = -2;
        return 0;
    }

    // EEPROM - lesen
    if ( fread( m_eep, sizeof(unsigned char), EEP_SIZE_SU, fp ) != EEP_SIZE_SU )
    {
        *error = -3;
        res = 0;
        goto error_label;
    }

    // Dateiheader lesen.
    if ( fread( m_header, sizeof(unsigned char), maxHeaderSize, fp ) != maxHeaderSize )
    {
        *error = -3;
        res = 0;
        goto error_label;
    }

    // Headerdaten pruefen, ggf. Anpassungen durchfuehren.
    if ( !checkHeader() )
    {
        // Header hat CRC Fehler.
        *error = -4;
        res = 0;
        goto error_label;
    }

    if ( IsHeader() )
    {
        // Mindestversion des Studio pruefen.
        if ( !(flags & FILESTUDIOHEADER_IGNORE_MIN_STUDIO_VERSION) && m_fileHeader->minStudioVersion )
        {
            CDFSoftwarebuildVersion minVersion( m_fileHeader->minStudioVersion );
            if ( minVersion > m_studioVersion )
            {
                *error = -5;
                res = 0;
                goto error_label;
            }
        }
        // Sollteversion des Studio pruefen?
        if ( !(flags & FILESTUDIOHEADER_IGNORE_SHOULD_STUDIO_VERSION) && !(m_fileHeader->flags & FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_STUDIO_VERSION) && m_fileHeader->shouldStudioVersion )
        {
            CDFSoftwarebuildVersion shouldVersion( m_fileHeader->shouldStudioVersion );
            if ( shouldVersion > m_studioVersion )
            {
                *error = -6;
                res = 0;
                goto error_label;
            }
        }

        // Mindestversion der Library pruefen.
        if ( !(flags & FILESTUDIOHEADER_IGNORE_MIN_LIBRARY_VERSION) && m_fileHeader->minLibraryVersion )
        {
            CDFSoftwarebuildVersion minVersion( m_fileHeader->minLibraryVersion );
            if ( minVersion > m_libraryVersion )
            {
                *error = -7;
                res = 0;
                goto error_label;
            }
        }
        // Sollteversion der Library pruefen?
        if ( !(flags & FILESTUDIOHEADER_IGNORE_SHOULD_LIBRARY_VERSION) && !(m_fileHeader->flags & FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_LIBRARY_VERSION) && m_fileHeader->shouldLibraryVersion )
        {
            CDFSoftwarebuildVersion shouldVersion( m_fileHeader->shouldLibraryVersion );
            if ( shouldVersion > m_libraryVersion )
            {
                *error = -8;
                res = 0;
                goto error_label;
            }
        }
    }
    else
    {
        // Den Header im Standard wiederherstellen.
        SetDefaultHeader();
    }

    // Setupgroesse
    sizeSetup = *(unsigned int*)&m_eep[P_FLASH_SIZE_SU];
    m_flashSizeSU2 = 0;
    ChangeOrderLong( (unsigned char*)&sizeSetup );
    // Ermittlung der Groesse des zu lesenden Speicherblocks fuer Flash.
    if ( (*(unsigned short*)&m_eep[P_FILE_VERSION] == 0xDF02) && (sizeSetup > FLASH_SIZE_SU) )
        m_flashSizeSU2 = FLASH_SIZE_SU2;

    // Flashbereich - lesen
    if ( fread( m_flash, sizeof(unsigned char), FLASH_SIZE_SU + m_flashSizeSU2, fp ) != (FLASH_SIZE_SU + m_flashSizeSU2) )
    {
        *error = -3;
        res = 0;
        goto error_label;
    }

    // ListFile - lesen
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        fread( buf, sizeof(unsigned char), 1, fp ); // ein byte Laenge
        if ( buf[0] == 0xFF )
        {
            fread( buf, sizeof(unsigned char), 2, fp ); // zwei byte Laenge
            length = *(unsigned short*)buf;

            listFileSize += 3;
        }
        else
        {
            length = buf[0];
            listFileSize++;
        }
        listFileSize += length;

        if ( fread(m_listFileName[i], sizeof(unsigned char), length, fp ) != length )
        {
            *error = -3;
            res = 0;
            goto error_label;
        }
    }

    // Konfiguration - lesen
    if ( fread( &m_config, sizeof(unsigned char), sizeof(StructConfig), fp ) != sizeof(StructConfig) )
    {
        *error = -3;
        res = 0;
        goto error_label;
    }


    // Sind ab aktueller Postition noch 16kByte erweiterter Speicher vorhanden?
    if ( (unsigned int)st.st_size >= 
        (EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(StructConfig) + EXTENDED_SIZE_SU) )
    {
        // Erweiterter Bereich - lesen: 16kByte Extended Flash
        if ( fread( m_flashExtended, sizeof(unsigned char), EXTENDED_SIZE_SU, fp ) != EXTENDED_SIZE_SU )
        {
            *error = -3;
            res = 0;
            goto error_label;
        }

        // Pruefen der Extended - Struktur
        StructExtendedFlash extBegin = *(StructExtendedFlash*)m_flashExtended;

        // existiert?
        if ( extBegin.check != EXTENDED_CHECK )
        {
            *error = -9;
            res = 0;
            goto error_label;
        }
    } 
    else
    {
        *error = -9;
        res = 0;
        goto error_label;
    }

    // Sind noch ListFile fuer Zutrittskontrolle vorhanden?
    if ( (unsigned int)st.st_size >= 
        (EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(StructConfig) + EXTENDED_SIZE_SU + 4) )
    {
        // ListFile - lesen
        for ( i = 0; i < 4; i++ )
        {
            fread( buf, sizeof(unsigned char), 1, fp ); // ein byte laenge
            if ( buf[0] == 0xFF )
            {
                fread( buf, sizeof(unsigned char), 2, fp ); // zwei byte laenge
                length = *(unsigned short*)buf;

                entranceListFileSize += 3;
            }
            else
            {
                length = buf[0];
                entranceListFileSize++;
            }
            entranceListFileSize += length;

            if ( fread( m_listEntranceFileName[i], sizeof(unsigned char), length, fp ) != length )
            {
                *error = -3;
                res = 0;
                goto error_label;
            }
        }

        // Loeschen, wenn ZK nicht aktiviert.
        if ( m_config.ENABLE_ENTRANCE == 0 )
        {
            memset( m_listEntranceFileName, 0, sizeof(m_listEntranceFileName) );
            // CRC von Pruefung ausschliessen.
            m_fileHeader->crcListEntranceFile = 0;
            // CRC ungueltig.
            m_fileHeader->crc = 0;
        }
    }
    else
    {
        *error = -9;
        res = 0;
        goto error_label;
    }

    // Sind noch ListFile fuer Zutrittskontrolle 2 vorhanden?
    if ( (unsigned int)st.st_size >= 
        (EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(StructConfig) + EXTENDED_SIZE_SU + entranceListFileSize + MAX_DATENSATZ_BESCHREIBUNG) )
    {
        // ListFile - lesen
        for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
        {
            fread( buf, sizeof(unsigned char), 1, fp ); // ein byte laenge
            if ( buf[0] == 0xFF )
            {
                fread( buf, sizeof(unsigned char), 2, fp ); // zwei byte laenge
                length = *(unsigned short*)buf;

                entrance2ListFileSize += 3;
            }
            else
            {
                length = buf[0];
                entrance2ListFileSize++;
            }
            entrance2ListFileSize += length;

            if ( fread( m_listEntrance2FileName[i], sizeof(unsigned char), length, fp ) != length )
            {
                *error = -3;
                res = 0;
                goto error_label;
            }
        }

        // Loeschen, wenn ZK nicht aktiviert.
        if ( m_config.ENABLE_ENTRANCE2 == 0 )
        {
            memset( m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName) );
            // CRC von Pruefung ausschliessen.
            m_fileHeader->crcListEntrance2File = 0;
            // CRC ungueltig.
            m_fileHeader->crc = 0;
        }
    }
    else
    {
        /*
        *error = -9;
        res = 0;
        goto error_label;
        */
        // (BO)Bugfix: 2012_03_19
        // Diese Dateiangaben muessen noch nicht verfuegbar sein.
        // Es koennten Dateien fuer AEIII+ ueber die Library geladen werden!
    }

    // Sind noch ExtendedFilePath fuer Zusatzdateien vorhanden?
    if ( (unsigned int)st.st_size >= 
        (EEP_SIZE_SU + maxHeaderSize + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(StructConfig) + EXTENDED_SIZE_SU + entranceListFileSize + entrance2ListFileSize + MAX_DATENSATZ_BESCHREIBUNG) )
    {
        // ListFile - lesen
        for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
        {
            fread( buf, sizeof(unsigned char), 1, fp ); // ein byte laenge
            if ( buf[0] == 0xFF )
            {
                fread( buf, sizeof(unsigned char), 2, fp ); // zwei byte laenge
                length = *(unsigned short*)buf;

                extendedFilePathSize += 3;
            }
            else
            {
                length = buf[0];
                extendedFilePathSize++;
            }
            extendedFilePathSize += length;

            if ( fread( m_strExtendedFilePath[i], sizeof(unsigned char), length, fp ) != length )
            {
                *error = -3;
                res = 0;
                goto error_label;
            }
        }
    }
    else
    {
        // Diese Dateiangaben muessen noch nicht verfuegbar sein.
        // Es koennen Dateien ab DatafoxStudioIV - Version 04.01.01.10 geladen werden!
    }

    if ( IsHeader() )
    {
        // CRC der Dateibereiche pruefen
        if ( m_fileHeader->crcEep && m_fileHeader->crcEep != GetCrcEep() )
        {
            *error = -10;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcEep = 0;

        if ( m_fileHeader->crcEepExtended && m_fileHeader->crcEepExtended != GetCrcEepExtended() )
        {
            *error = -11;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcEepExtended = 0;

        if ( m_fileHeader->crcFlash && m_fileHeader->crcFlash != GetCrcFlash() )
        {
            *error = -12;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcFlash = 0;

        if ( m_fileHeader->crcListFile && m_fileHeader->crcListFile != GetCrcListFile() )
        {
            *error = -13;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcListFile = 0;

        if ( m_fileHeader->crcConfig && m_fileHeader->crcConfig != GetCrcConfig() )
        {
            *error = -14;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcConfig = 0;

        if ( m_fileHeader->crcFlashExtended && m_fileHeader->crcFlashExtended != GetCrcFlashExtended() )
        {
            *error = -15;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcFlashExtended = 0;

        if ( m_fileHeader->crcListEntranceFile && m_fileHeader->crcListEntranceFile != GetCrcListEntranceFile() )
        {
            *error = -16;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcListEntranceFile = 0;

        if ( m_fileHeader->crcListEntrance2File && m_fileHeader->crcListEntrance2File != GetCrcListEntrance2File() )
        {
            *error = -17;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcListEntrance2File = 0;

        if ( m_fileHeader->crcListExtendedFile && m_fileHeader->crcListExtendedFile != GetCrcListExtendedFile() )
        {
            *error = -18;
            res = 0;
            goto error_label;
        }
        m_fileHeader->crcListExtendedFile = 0;
    }
    // Die Werte fuer Dateinamen und Zeitstempel der Modifizierung eintragen.
    // Dateinamen extrahieren
    fName = (char*)&fileName[strlen( fileName )];
    while ( fName > fileName )
    {
        // Maskierte Leerzeichen (unix)
        if ( *(fName - 1) == '\\' && *fName == ' ' )
        {

        }
        else if( *(fName - 1) == '\\' )
        {
            // Pfadbestandteil vor Dateinamen (windows)
            break;
        }
        else if ( *(fName - 1) == '/' )
        {
            // Pfadbestandteil vor Dateinamen (unix)
            break;
        } 

        fName--;
    }

    ptm = localtime( &st.st_mtime );
    sprintf( mTimeStamp, "%02i%02i-%02i-%02i %02i:%02i:%02i", 
        (ptm->tm_year + 1900) / 100, (ptm->tm_year + 1900) % 100, ptm->tm_mon + 1, ptm->tm_mday,
        ptm->tm_hour, ptm->tm_min, ptm->tm_sec );

    // Eintragen in der Extended - Struktur
    extBegin = *(StructExtendedFlash*)m_flashExtended;
    if ( extBegin.check == EXTENDED_CHECK && extBegin.offsetExtendedStruct )
    {
        StructExtended *extended = (StructExtended*)&m_flashExtended[extBegin.offsetExtendedStruct];
        strncpy( extended->fileName, fName, sizeof(extended->fileName) );
        // Nullterminierung sicherstellen
        extended->fileName[sizeof(extended->fileName) - 1] = 0;
        strcpy( extended->fileTimeStamp, mTimeStamp );
    }

error_label:
    if ( fp )
        fclose( fp );

    return res;
}
#include <errno.h>

int FileStudio::Save( const char *fileName, int *error )
{
    int res = 1;
    int i;
    unsigned int sizeSetup;

    *error = 0;
    FILE *fp = fopen( fileName, "wb" );
    if ( fp == NULL )
        return 0;

    // Groesse der Setupdaten eintragen
    sizeSetup = GetFlashSetupSize();
    *(unsigned int*)&m_eep[P_FLASH_SIZE_SU] = sizeSetup;
    ChangeOrderLong(&m_eep[P_FLASH_SIZE_SU]);

    // Ermittlung der Groesse des zu schreibenden Speicherblocks fuer Flash.
    if ( sizeSetup > FLASH_SIZE_SU )
    {
        m_flashSizeSU2 = FLASH_SIZE_SU2;
        *(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF02;
    }
    else
    {
        m_flashSizeSU2 = 0;
        *(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF01;
    }

    // EEPROM - schreiben
    if ( fwrite( m_eep, sizeof(unsigned char), EEP_SIZE_SU, fp ) != EEP_SIZE_SU )
    {
        res = 0;
        goto error_label;
    }

    // Headerdaten erstellen, ggf. Anpassungen durchfuehren.
    if ( !buildHeader() )
    {
        res = 0;
        goto error_label;
    }

    // Dateiheaderbereich schreiben
    if ( fwrite( m_header, sizeof(unsigned char), EEP_SIZE - EEP_SIZE_SU, fp ) != (EEP_SIZE - EEP_SIZE_SU) )
    {
        res = 0;
        goto error_label;
    }

    // Flashbereich - schreiben
    if ( fwrite( m_flash, sizeof(unsigned char), FLASH_SIZE_SU + m_flashSizeSU2, fp ) != (FLASH_SIZE_SU + m_flashSizeSU2) )
    {
        res = 0;
        goto error_label;
    }

    // ListFile - schreiben
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listFileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listFileName[i] );
            if ( fwrite( len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listFileName[i] );
            if ( fwrite( &len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }

        // String rausschreiben
        if ( fwrite( m_listFileName[i], sizeof(unsigned char), strlen( m_listFileName[i] ), fp ) != strlen( m_listFileName[i] ) )
        {
            res = 0;
            goto error_label;
        }
    }

    // Konfiguration - schreiben
    if ( fwrite( &m_config, sizeof(unsigned char), sizeof(StructConfig), fp ) != sizeof(StructConfig) )
    {
        res = 0;
        goto error_label;
    }

    // Erweiterter Speicher - schreiben
    if ( fwrite( m_flashExtended, sizeof(unsigned char), EXTENDED_SIZE_SU, fp ) != EXTENDED_SIZE_SU )
    {
        res = 0;
        goto error_label;
    }

    // Zutrittskontrolle - ListFile - schreiben
    for ( i = 0; i < 4; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listEntranceFileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listEntranceFileName[i] );
            if ( fwrite( len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listEntranceFileName[i] );
            if ( fwrite( &len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }

        // String rausschreiben
        if ( fwrite( m_listEntranceFileName[i], sizeof(unsigned char), strlen( m_listEntranceFileName[i] ), fp ) != strlen( m_listEntranceFileName[i] ) )
        {
            res = 0;
            goto error_label;
        }
    }
    // Zutrittskontrolle - ListFile - schreiben
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listEntrance2FileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listEntrance2FileName[i] );
            if ( fwrite( len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listEntrance2FileName[i] );
            if ( fwrite( &len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }

        // String rausschreiben
        if ( fwrite( m_listEntrance2FileName[i], sizeof(unsigned char), strlen( m_listEntrance2FileName[i] ), fp ) != strlen( m_listEntrance2FileName[i] ) )
        {
            res = 0;
            goto error_label;
        }
    }

    // Zusatzpfadangaben - schreiben
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_strExtendedFilePath[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_strExtendedFilePath[i] );
            if ( fwrite( len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_strExtendedFilePath[i] );
            if ( fwrite( &len, sizeof(unsigned char), sizeof(len), fp ) != sizeof(len) )
            {
                res = 0;
                goto error_label;
            }
        }

        // String rausschreiben
        if ( fwrite( m_strExtendedFilePath[i], sizeof(unsigned char), strlen( m_strExtendedFilePath[i] ), fp ) != strlen( m_strExtendedFilePath[i] ) )
        {
            res = 0;
            goto error_label;
        }
    }

error_label:
    if ( fp )
        fclose( fp );

    return res;
}

//########################
// CRC der Dateibereiche
//########################

///< CRC ueber Eep-Bereich.
unsigned int FileStudio::GetCrcEep()
{
    return GetCRC32( m_eep, EEP_SIZE_SU );
}

///< CRC ueber EepExtended-Bereich.
unsigned int FileStudio::GetCrcEepExtended()
{
    return GetCRC32( &m_header[m_fileHeader->bugfixSize], EEP_SIZE - EEP_SIZE_SU - m_fileHeader->bugfixSize );
}

///< CRC ueber Flash-Bereich.
unsigned int FileStudio::GetCrcFlash()
{
    // Groesse der Setupdaten
    unsigned int sizeSetup = GetFlashSetupSize();

    // Ermittlung der Groesse des zu beachtenden Speicherblocks fuer Flash.
    if ( sizeSetup > FLASH_SIZE_SU )
        m_flashSizeSU2 = FLASH_SIZE_SU2;

    return GetCRC32( m_flash, FLASH_SIZE_SU + m_flashSizeSU2 );
}

///< CRC ueber die Listendateiangaben.
unsigned int FileStudio::GetCrcListFile()
{
    unsigned int crc = 0;
    int i;
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listFileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listFileName[i] );
            crc = GetCRC32( len, sizeof(len), crc, 0 );
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listFileName[i] );
            crc = GetCRC32( &len, sizeof(len), crc, 0 );
        }

        // String rausschreiben
        crc = GetCRC32( (unsigned char*)m_listFileName[i], strlen( m_listFileName[i] ), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)) );
    }

    return crc;
}

///< CRC ueber die Config-Struktur.
unsigned int FileStudio::GetCrcConfig()
{
    return GetCRC32( (unsigned char*)&m_config, sizeof(StructConfig) );
}

///< CRC ueber FlashExtended-Bereich.
unsigned int FileStudio::GetCrcFlashExtended()
{
    return GetCRC32( m_flashExtended, EXTENDED_SIZE_SU );
}

///< CRC ueber die Listendateiangaben der ZK1.
unsigned int FileStudio::GetCrcListEntranceFile()
{
    unsigned int crc = 0;
    int i;
    for ( i = 0; i < 4; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listEntranceFileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listEntranceFileName[i] );
            crc = GetCRC32( len, sizeof(len), crc, 0 );
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listEntranceFileName[i] );
            crc = GetCRC32( &len, sizeof(len), crc, 0 );
        }

        // String rausschreiben
        crc = GetCRC32( (unsigned char*)m_listEntranceFileName[i], strlen( m_listEntranceFileName[i] ), crc, (i == (4 - 1)) );
    }

    return crc;
}

///< CRC ueber die Listendateiangaben der ZK2.
unsigned int FileStudio::GetCrcListEntrance2File()
{
    unsigned int crc = 0;
    int i;
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_listEntrance2FileName[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_listEntrance2FileName[i] );
            crc = GetCRC32( len, sizeof(len), crc, 0 );
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_listEntrance2FileName[i] );
            crc = GetCRC32( &len, sizeof(len), crc, 0 );
        }

        // String rausschreiben
        crc = GetCRC32( (unsigned char*)m_listEntrance2FileName[i], strlen( m_listEntrance2FileName[i] ), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)) );
    }

    return crc;
}

///< CRC ueber die Zusaetzlichen Listendateiangaben.
unsigned int FileStudio::GetCrcListExtendedFile()
{
    unsigned int crc = 0;
    int i;
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
        if ( strlen( m_strExtendedFilePath[i] ) > 0xFF )
        {
            unsigned char len[3];
            len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
            *(unsigned short*)&len[1] = (unsigned short)strlen( m_strExtendedFilePath[i] );
            crc = GetCRC32( len, sizeof(len), crc, 0 );
        }
        else
        {
            unsigned char len = (unsigned char)strlen( m_strExtendedFilePath[i] );
            crc = GetCRC32( &len, sizeof(len), crc, 0 );
        }

        // String rausschreiben
        crc = GetCRC32( (unsigned char*)m_strExtendedFilePath[i], strlen( m_strExtendedFilePath[i] ), crc, (i == (MAX_DATENSATZ_BESCHREIBUNG - 1)) );
    }

    return crc;
}

// Die zu verwendende Version der Datei
int FileStudio::NeededVersion()
{
    if (GetFlashSetupSize() > FLASH_SIZE_SU)
        return 2;

    return 1;
}

void FileStudio::ChangeOrderLong(unsigned char *Buf)
{
    unsigned char h;

    h = Buf[0];
    Buf[0] = Buf[3];
    Buf[3] = h;

    h = Buf[1];
    Buf[1] = Buf[2];
    Buf[2] = h;
}

// Groesse der Setupdaten im Flash ermitteln
unsigned int FileStudio::GetFlashSetupSize()
{
    // ##############################################################
    // Pointer sind direkte Adressen die im AEIII+:
    //   1. Im Motorolla - Format vorliegen.
    //   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

    // Pointer auf ListenBeschreibungen ermitteln
    unsigned int pLB = *(unsigned int*)&m_eep[P_LIST_STRUCT];
    ChangeOrderLong( (unsigned char*)&pLB );

    pLB -= FLASH_KONFIG_START;

    // Pointerarray der Listenbeschreibungen durchschreiten
    int i;
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Nullpointer suchen fuer Listenanzahl
        if ( *(unsigned int*)&m_flash[pLB + i * sizeof(unsigned int)] == 0ul )
            break;
    }

    // Liste gefunden?
    if ( i > 0 )
    {
        // Pointer auf letzte definierte Liste
        pLB = *(unsigned int*)&m_flash[pLB + (i - 1) * sizeof(unsigned int)];
        ChangeOrderLong( (unsigned char*)&pLB );

        pLB -= FLASH_KONFIG_START;

        StructDatensatzBeschreibung *pLBStruct = (StructDatensatzBeschreibung*)&m_flash[pLB];

        // Pointer auf Feldarray der Liste
        unsigned int pFeldArray = pLBStruct->P;
        ChangeOrderLong((unsigned char*)&pFeldArray);

        pFeldArray -= FLASH_KONFIG_START;

        // Felder vorhanden?
        if ( pLBStruct->Felder )
        {
            // Pointer auf letztes Feld der Listenbeschreibung ermitteln.
            unsigned int pFeld = *(unsigned int*)&m_flash[pFeldArray + (pLBStruct->Felder - 1) * sizeof(unsigned int)];
            ChangeOrderLong((unsigned char*)&pFeld);

            pFeld -= FLASH_KONFIG_START;

            // Feldadresse + eigene Groesse bildet FlashSetupSize
            return pFeld + sizeof(StructDatensatzFeld);
        }


        // Keine Felder definiert!
        return pFeldArray;
    }

    // Keine Listenbeschreibung definiert!
    // Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
    // um bei einer Uebertragung diese zu loeschen.
    return (pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned int));
}

// Groesse des Erweiterten Setupbereichs ermitteln
unsigned int FileStudio::GetExtendedFlashSetupSize()
{
    // Erweiterten Struktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_flashExtended;

    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK)
        return 0;

    return extBegin.offsetFreeScope;
}

// Groesse der Listen- oder Datensatzbeschreibungen im Flash ermitteln
// \param lists[in] Gibt an ob die Groesse der Listen- oder Datensatzbeschreibungen ermittelt werden sollen.
unsigned int FileStudio::GetTablesSize( bool lists )
{
    // ##############################################################
    // Pointer sind direkte Adressen die im AEIII+:
    //   1. Im Motorolla - Format vorliegen.
    //   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

    // Pointer auf Listen- oder DatensatzBeschreibungen ermitteln
    unsigned int pTables = *(unsigned int*)&m_eep[(lists) ? P_LIST_STRUCT : P_DAT_STRUCT];
    ChangeOrderLong( (unsigned char*)&pTables );

    pTables -= FLASH_KONFIG_START;

    // Pointerarray der Listenbeschreibungen durchschreiten
    int i;
    for ( i = 0; i < MAX_DATENSATZ_BESCHREIBUNG; i++ )
    {
        // Nullpointer suchen fuer letzte Tabelle
        if ( *(unsigned int*)&m_flash[pTables + i * sizeof(unsigned int)] == 0 )
            break;
    }

    // Tabelle gefunden?
    if ( i > 0 )
    {
        // Pointer auf die letzte definierte Tabelle
        pTables = *(unsigned int*)(&m_flash[pTables + (i - 1) * sizeof(unsigned int)]);
        ChangeOrderLong((unsigned char*)&pTables);

        pTables -= FLASH_KONFIG_START;

        StructDatensatzBeschreibung *pTableStruct = (StructDatensatzBeschreibung*)&m_flash[pTables];

        // Pointer auf Feldarray der Tabelle
        unsigned int pFeldArray = pTableStruct->P;
        ChangeOrderLong((unsigned char*)&pFeldArray);

        pFeldArray -= FLASH_KONFIG_START;

        // Felder vorhanden?
        if ( pTableStruct->Felder )
        {
            // Pointer auf letztes Feld der Listenbeschreibung ermitteln.
            unsigned int pFeld = *(unsigned int*)&m_flash[pFeldArray + (pTableStruct->Felder - 1) * sizeof(unsigned int)];
            ChangeOrderLong((unsigned char*)&pFeld);

            pFeld -= FLASH_KONFIG_START;

            // Feldadresse + eigene Groesse bilden Endadresse
            return ((pFeld + sizeof(StructDatensatzFeld)) - pTables);
        }


        // Keine Felder definiert!
        return (pFeldArray - pTables);
    }

    // Keine Tabellen definiert!
    return 0;
}

// MD5 ueber den Dateiheader berechnen.
char *FileStudio::GetMD5Header()
{
    if ( !IsHeader() )
        return 0;

    return MD5Bin( m_header, m_fileHeader->size );
}

// MD5 ueber Listen- oder Datensatzbeschreibungen im Flash berechnen.
// \param lists[in] Gibt an ob der MD5 der Listen- oder Datensatzbeschreibungen ermittelt werden soll.
char *FileStudio::GetMD5Tables( bool lists )
{
    // ##############################################################
    // Pointer sind direkte Adressen die im AEIII+:
    //   1. Im Motorolla - Format vorliegen.
    //   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

    // Pointer auf Listen- oder DatensatzBeschreibungen ermitteln
    unsigned int pTables = *(unsigned int*)&m_eep[(lists) ? P_LIST_STRUCT : P_DAT_STRUCT];
    ChangeOrderLong( (unsigned char*)&pTables );

    pTables -= FLASH_KONFIG_START;

    // Pointerarray der Listenbeschreibungen durchschreiten
    int size = GetTablesSize( lists );

    // Tabelle gefunden?
    if ( size > 0 )
        return MD5Bin( &m_flash[pTables], size );

    // Keine Tabellen definiert!
    return 0;
}

int FileStudio::ValidFirmwareVersion( CDFSoftwarebuildVersion &fwVersion )
{
    if ( IsHeader() )
    {
        // Pruefung auf 04.01.xx
        if ( fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00" )
        {
            // Unsupported der Firmware 04.01.xx.yy pruefen.
            if ( m_fileHeader->minFirmware1Version == FILESTUDIOHEADER_VERSION_UNSUPPORTED )
                return -1;

            // Mindestversion der Firmware 04.01.xx.yy pruefen.
            if ( m_fileHeader->minFirmware1Version )
            {
                CDFSoftwarebuildVersion minVersion( m_fileHeader->minFirmware1Version );
                if ( minVersion > fwVersion )
                    return -2;
            }
            // Sollteversion des der Firmware 04.01.xx.yy pruefen?
            if ( !(m_fileHeader->flags & FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE1_VERSION) && m_fileHeader->shouldFirmware1Version )
            {
                CDFSoftwarebuildVersion shouldVersion( m_fileHeader->shouldFirmware1Version );
                if ( shouldVersion > fwVersion )
                    return -3;
            }
        }
        else if ( fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00" )
        {

            // Unsupported der Firmware 04.02.xx.yy pruefen.
            if ( m_fileHeader->minFirmware2Version == FILESTUDIOHEADER_VERSION_UNSUPPORTED )
                return -4;

            // Mindestversion der der Firmware 04.02.xx.yy pruefen.
            if ( m_fileHeader->minFirmware2Version )
            {
                CDFSoftwarebuildVersion minVersion( m_fileHeader->minFirmware2Version );
                if ( minVersion > fwVersion )
                    return -5;
            }
            // Sollteversion der der Firmware 04.02.xx.yy pruefen?
            if ( !(m_fileHeader->flags & FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE2_VERSION) && m_fileHeader->shouldFirmware2Version )
            {
                CDFSoftwarebuildVersion shouldVersion( m_fileHeader->shouldFirmware2Version );
                if ( shouldVersion > fwVersion )
                    return -6;
            }
        }
        else if ( fwVersion >= "04.03.00.00" && fwVersion < "04.04.00.00" )
        {
            // Unsupported der Firmware 04.03.xx.yy pruefen.
            if ( m_fileHeader->minFirmware3Version == FILESTUDIOHEADER_VERSION_UNSUPPORTED )
                return -8;

            // Mindestversion der der Firmware 04.03.xx.yy pruefen.
            if ( m_fileHeader->minFirmware3Version )
            {
                CDFSoftwarebuildVersion minVersion( m_fileHeader->minFirmware3Version );
                if ( minVersion > fwVersion )
                    return -9;
            }
            // Sollteversion der der Firmware 04.03.xx.yy pruefen?
            if ( !(m_fileHeader->flags & FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE3_VERSION) && m_fileHeader->shouldFirmware3Version )
            {
                CDFSoftwarebuildVersion shouldVersion( m_fileHeader->shouldFirmware3Version );
                if ( shouldVersion > fwVersion )
                    return -10;
            }
        }
        else
        {
            // Die Firmwareversion wird nicht unterstützt.
            return -7;
        }

        // Firmwareversion wird unterstuetzt.
        return 1;
    }

    // Es liegt kein Dateiheader vor!
    return 0;
}
